import java.awt.*;
import java.awt.geom.*;

import javax.swing.*;

/**
 * A user avatar.
 */
public class JuGuy {
	
	/**
	 * You can change the following constants.
	 */
	static final Image guy_bg = new ImageIcon(JuGuy.class.getResource("guy1.png")).getImage();
	static final Image guy_fg = new ImageIcon(JuGuy.class.getResource("guy2.png")).getImage();
	static final Image guy_icon = new ImageIcon(JuGuy.class.getResource("guy_icon.png")).getImage();
	static final int width = guy_bg.getWidth(null);
	static final int height = guy_bg.getHeight(null);
	static final int icon_width = guy_icon.getWidth(null);
	static final int icon_height = guy_icon.getHeight(null);
	static final Composite delete_opacity = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.15f);
	
	private final JuTable parent; // for repaint
	float x, y, orientation;
	private AffineTransform transform = null;
	boolean visible = true;
	
	public JuGuy(JuTable parent) {
		this.parent = parent;
	}

	public void setLocation(float x, float y) {
		this.x = x;
		this.y = y;
		transform = null;
	}
	
	protected void setOrientation(float angle) {
		orientation = angle;
		transform = null;
	}
	
	public Rectangle getBounds() {
		return new Rectangle((int)x - width/2 - 30, (int)y - width/2 - 30, width+70, width+50);
	}
	
	public AffineTransform getTransform() {
		if (transform == null) {
			transform = AffineTransform.getTranslateInstance(x, y);
			transform.rotate(orientation);
		}
		return transform;
	}
	
	public void repaint() {
		parent.repaint(getBounds());
	}
	
	public void paint_bg(Graphics2D g) {
		Composite oldcomposite = g.getComposite();
		if (!visible)
			g.setComposite(delete_opacity);
		AffineTransform ot = g.getTransform();
		g.transform(getTransform());
		g.drawImage(guy_bg, -width/2, -height/2, null);
		g.setTransform(ot);
		g.setComposite(oldcomposite);
	}
	
	public void paint_fg(Graphics2D g) {
		Composite oldcomposite = g.getComposite();
		if (!visible)
			g.setComposite(delete_opacity);
		AffineTransform ot = g.getTransform();
		g.transform(getTransform());
		g.drawImage(guy_fg, -width/2, -height/2,null);
		g.setTransform(ot);
		g.setComposite(oldcomposite);
	}
	
	public void drag(Point2D p0, Point2D p1) {
		repaint();
		x += p1.getX() - p0.getX();
		y += p1.getY() - p0.getY();
		transform = null;
		repaint();
	}

	public void rotate(Point2D p0, Point2D p1) {
		repaint();
		float delta = (float)(p1.getX() - p0.getX())/60;
		setOrientation(orientation + delta);
		repaint();
	}
}
